(document:surround "/std/frame")
(document:insert "/std/functions")

(document:envelop with-translation _ "alterator-ldap-users")

;;; Functions

(define (read-user name)
  (let ((data (woo-read-first "/ldap-users/" 'name name)))
    (passwd_auto value (woo-get-option data 'passwd_auto))
    (user-gecos value (woo-get-option data 'gecos))
    (user-home value (woo-get-option data 'home))
    (user-shell value (woo-get-option data 'shell))
    (user-allow-su value (woo-get-option data 'allow_su))))

(define (list-user)
  (user-shell enumref "/ldap-users/avail_shell")
  (user-list enumref "/ldap-users")

  (and (positive? (user-list count)) (user-list current 0))
  (read-user ""))

(define (delete-user)
  (and-let* ((name (user-list value)))
    (catch/message
      (lambda()
         (if (eq? 'yes (document:popup-warning
		       (string-append (_ "Do you really want to delete user ")
				      name
				      " ?")
		       'no
		       'yes))
	      (begin
	        (woo-delete "/ldap-users/" 'name name)
	        (list-user)))))))

(define (new-user)
  (catch/message
    (lambda()
      (woo-new "/ldap-users" 'name (new_name value))
      (list-user))))

(define (write-user)
  (catch/message
    (lambda()
      (woo-write  "/ldap-users/"
		  'name (user-list value)
		  'home (user-home value)
		  'gecos (user-gecos value)
		  'shell (user-shell value)
		  'allow_su (user-allow-su value)
		  'passwd_1 (passwd_1 value)
		  'passwd_2 (passwd_2 value)
		  'passwd_auto (passwd_auto value)
		  'auto (auto value)))))

;;; UI
(gridbox
  columns "100"
  align "top"
  margin 10

  (label)
  (hbox align "left"
    (label text (_ "New account:") name "name")
    (document:id new_name (edit))
    (button text (_ "Create") (when clicked (new-user))))

  (separator)
  (splitbox
    columns "30;70"
    (document:id user-list
		 (listbox ;;header (vector (_ "Accounts"))
		          align "top"
			  (when selected 
			    (catch/message
			      (lambda() (read-user (user-list value)))))))
    (gridbox
      columns "0;100"
      ;;
      (label text (_ "Comment:") align "right")
      (document:id user-gecos (edit))
      ;;
      (label text (_ "Home directory:") align "right")
      (document:id user-home (edit))
      ;;
      (label text (_ "Shell:") align "right")
      (document:id user-shell (combobox))
      ;;
      (spacer)
		
      ;;
      (label colspan 2)

      ;;
      (label text (_ "Password:") align "top;right")
      (gridbox
	columns "100;0"
	(document:id auto (checkbox text (_ "Generate automatically") name "auto"))
	(spacer)
	(document:id passwd_1 (edit name "passwd_1" echo "stars" visibility #f))
	(label name "passwd_1" text (small (_ "(enter passphrase)")) visibility #f)
	(document:id passwd_2 (edit name "passwd_2" echo "stars" visibility #f))
	(label name "passwd_1" text (small (_ "(repeat passphrase)")) visibility #f)
	(document:id passwd_auto (edit name "passwd_auto" visibility #f))
	(button text (_ "Generate") name "passwd_auto" visibility #f (when clicked (read-user (user-list value))))
	(label colspan 2 name "passwd_auto" visibility #f))

      ;;
      (label colspan 2)

      ;;
      (spacer)
      (hbox align "left"
            (button text (_ "Apply") (when clicked (write-user)))
	    (label)
	    (button text (_ "Delete user") (when clicked (delete-user)))))))

;;; Logic
(effect-show "passwd_1" "auto" #f)
(effect-show "passwd_2" "auto" #f)
(effect-show "passwd_auto" "auto" #t)

(document:root
  (when loaded
    (catch/message
      (lambda()
	(init-effect)
	(list-user)))))
